@@ -3,7 +3,7 @@
 
 import copy
 from typing import (
-    Any, Dict, Iterable, Iterator, List, Optional, Set, Union,
+    TYPE_CHECKING, Any, Dict, Iterable, Iterator, List, Optional, Set, Union,
 )
 
 from amundsen_common.utils.atlas import (
@@ -27,19 +27,23 @@ from databuilder.models.atlas_relationship import AtlasRelationship
 from databuilder.models.atlas_serializable import AtlasSerializable
 from databuilder.models.badge import Badge, BadgeMetadata
 from databuilder.models.cluster import cluster_constants
+from databuilder.models.description_metadata import (  # noqa: F401
+    DESCRIPTION_NODE_LABEL, DESCRIPTION_NODE_LABEL_VAL, DescriptionMetadata,
+)
 from databuilder.models.graph_node import GraphNode
 from databuilder.models.graph_relationship import GraphRelationship
 from databuilder.models.graph_serializable import GraphSerializable
 from databuilder.models.schema import schema_constant
 from databuilder.models.table_serializable import TableSerializable
+
+if TYPE_CHECKING:
+    from databuilder.models.type_metadata import TypeMetadata
+
 from databuilder.serializers.atlas_serializer import (
     add_entity_relationship, get_entity_attrs, get_entity_relationships,
 )
 from databuilder.utils.atlas import AtlasRelationshipTypes, AtlasSerializedEntityOperation
 
-DESCRIPTION_NODE_LABEL_VAL = 'Description'
-DESCRIPTION_NODE_LABEL = DESCRIPTION_NODE_LABEL_VAL
-
 
 def _format_as_list(tags: Union[List, str, None]) -> List:
     if tags is None:
@@ -172,140 +176,6 @@ class TagMetadata(GraphSerializable, TableSerializable, AtlasSerializable):
             return None
 
 
-# TODO: this should inherit from ProgrammaticDescription in amundsen-common
-class DescriptionMetadata(GraphSerializable, AtlasSerializable):
-    DESCRIPTION_NODE_LABEL = DESCRIPTION_NODE_LABEL_VAL
-    PROGRAMMATIC_DESCRIPTION_NODE_LABEL = 'Programmatic_Description'
-    DESCRIPTION_KEY_FORMAT = '{description}'
-    DESCRIPTION_TEXT = 'description'
-    DESCRIPTION_SOURCE = 'description_source'
-
-    DESCRIPTION_RELATION_TYPE = 'DESCRIPTION'
-    INVERSE_DESCRIPTION_RELATION_TYPE = 'DESCRIPTION_OF'
-
-    # The default editable source.
-    DEFAULT_SOURCE = "description"
-
-    def __init__(self,
-                 text: Optional[str],
-                 source: str = DEFAULT_SOURCE,
-                 description_key: Optional[str] = None,
-                 start_label: Optional[str] = None,  # Table, Column, Schema
-                 start_key: Optional[str] = None,
-                 ):
-        """
-        :param source: The unique source of what is populating this description.
-        :param text: the description text. Markdown supported.
-        """
-        self.source = source
-        self.text = text
-        #  There are so many dependencies on Description node, that it is probably easier to just separate the rest out.
-        if self.source == self.DEFAULT_SOURCE:
-            self.label = self.DESCRIPTION_NODE_LABEL
-        else:
-            self.label = self.PROGRAMMATIC_DESCRIPTION_NODE_LABEL
-
-        self.start_label = start_label
-        self.start_key = start_key
-        self.description_key = description_key or self.get_description_default_key(start_key)
-
-        self._node_iter = self._create_node_iterator()
-        self._relation_iter = self._create_relation_iterator()
-
-    @staticmethod
-    def create_description_metadata(text: Union[None, str],
-                                    source: Optional[str] = DEFAULT_SOURCE,
-                                    description_key: Optional[str] = None,
-                                    start_label: Optional[str] = None,  # Table, Column, Schema
-                                    start_key: Optional[str] = None,
-                                    ) -> Optional['DescriptionMetadata']:
-        # We do not want to create a node if there is no description text!
-        if text is None:
-            return None
-        description_node = DescriptionMetadata(text=text,
-                                               source=source or DescriptionMetadata.DEFAULT_SOURCE,
-                                               description_key=description_key,
-                                               start_label=start_label,
-                                               start_key=start_key)
-        return description_node
-
-    def get_description_id(self) -> str:
-        if self.source == self.DEFAULT_SOURCE:
-            return "_description"
-        else:
-            return "_" + self.source + "_description"
-
-    def get_description_default_key(self, start_key: Optional[str]) -> Optional[str]:
-        return f'{start_key}/{self.get_description_id()}' if start_key else None
-
-    def get_node(self, node_key: str) -> GraphNode:
-        node = GraphNode(
-            key=node_key,
-            label=self.label,
-            attributes={
-                DescriptionMetadata.DESCRIPTION_SOURCE: self.source,
-                DescriptionMetadata.DESCRIPTION_TEXT: self.text
-            }
-        )
-        return node
-
-    def get_relation(self,
-                     start_node: str,
-                     start_key: str,
-                     end_key: str,
-                     ) -> GraphRelationship:
-        relationship = GraphRelationship(
-            start_label=start_node,
-            start_key=start_key,
-            end_label=self.label,
-            end_key=end_key,
-            type=DescriptionMetadata.DESCRIPTION_RELATION_TYPE,
-            reverse_type=DescriptionMetadata.INVERSE_DESCRIPTION_RELATION_TYPE,
-            attributes={}
-        )
-        return relationship
-
-    def create_next_node(self) -> Optional[GraphNode]:
-        # return the string representation of the data
-        try:
-            return next(self._node_iter)
-        except StopIteration:
-            return None
-
-    def create_next_relation(self) -> Optional[GraphRelationship]:
-        try:
-            return next(self._relation_iter)
-        except StopIteration:
-            return None
-
-    def _create_node_iterator(self) -> Iterator[GraphNode]:
-        if not self.description_key:
-            raise Exception('Required description node key cannot be None')
-        yield self.get_node(self.description_key)
-
-    def _create_relation_iterator(self) -> Iterator[GraphRelationship]:
-        if not self.start_label:
-            raise Exception('Required relation start node label cannot be None')
-        if not self.start_key:
-            raise Exception('Required relation start key cannot be None')
-        if not self.description_key:
-            raise Exception('Required relation end key cannot be None')
-        yield self.get_relation(
-            start_node=self.start_label,
-            start_key=self.start_key,
-            end_key=self.description_key
-        )
-
-    def create_next_atlas_relation(self) -> Union[AtlasRelationship, None]:
-        pass
-
-    def create_next_atlas_entity(self) -> Union[AtlasEntity, None]:
-        pass
-
-    def __repr__(self) -> str:
-        return f'DescriptionMetadata({self.source!r}, {self.text!r})'
-
-
 class ColumnMetadata:
     COLUMN_NODE_LABEL = 'Column'
     COLUMN_KEY_FORMAT = '{db}://{cluster}.{schema}/{tbl}/{col}'
@@ -338,10 +208,26 @@ class ColumnMetadata:
         formatted_badges = _format_as_list(badges)
         self.badges = [Badge(badge, 'column') for badge in formatted_badges]
 
+        # The following fields are populated by the ComplexTypeTransformer
+        self._column_key: Optional[str] = None
+        self._type_metadata: Optional[TypeMetadata] = None
+
     def __repr__(self) -> str:
         return f'ColumnMetadata({self.name!r}, {self.description!r}, {self.type!r}, ' \
                f'{self.sort_order!r}, {self.badges!r})'
 
+    def get_column_key(self) -> Optional[str]:
+        return self._column_key
+
+    def set_column_key(self, col_key: str) -> None:
+        self._column_key = col_key
+
+    def get_type_metadata(self) -> Optional['TypeMetadata']:
+        return self._type_metadata
+
+    def set_type_metadata(self, type_metadata: 'TypeMetadata') -> None:
+        self._type_metadata = type_metadata
+
 
 class TableMetadata(GraphSerializable, TableSerializable, AtlasSerializable):
     """
@@ -502,29 +388,7 @@ class TableMetadata(GraphSerializable, TableSerializable, AtlasSerializable):
                 yield tag_node
 
         for col in self.columns:
-            column_node = GraphNode(
-                key=self._get_col_key(col),
-                label=ColumnMetadata.COLUMN_NODE_LABEL,
-                attributes={
-                    ColumnMetadata.COLUMN_NAME: col.name,
-                    ColumnMetadata.COLUMN_TYPE: col.type,
-                    ColumnMetadata.COLUMN_ORDER: col.sort_order
-                }
-            )
-            yield column_node
-
-            if col.description:
-                node_key = self._get_col_description_key(col, col.description)
-                yield col.description.get_node(node_key)
-
-            if col.badges:
-                col_badge_metadata = BadgeMetadata(
-                    start_label=ColumnMetadata.COLUMN_NODE_LABEL,
-                    start_key=self._get_col_key(col),
-                    badges=col.badges)
-                badge_nodes = col_badge_metadata.get_badge_nodes()
-                for node in badge_nodes:
-                    yield node
+            yield from self._create_column_nodes(col)
 
         # Database, cluster, schema
         others = [
@@ -572,6 +436,35 @@ class TableMetadata(GraphSerializable, TableSerializable, AtlasSerializable):
             attributes=table_attributes
         )
 
+    def _create_column_nodes(self, col: ColumnMetadata) -> Iterator[GraphNode]:
+        column_node = GraphNode(
+            key=self._get_col_key(col),
+            label=ColumnMetadata.COLUMN_NODE_LABEL,
+            attributes={
+                ColumnMetadata.COLUMN_NAME: col.name,
+                ColumnMetadata.COLUMN_TYPE: col.type,
+                ColumnMetadata.COLUMN_ORDER: col.sort_order
+            }
+        )
+        yield column_node
+
+        if col.description:
+            node_key = self._get_col_description_key(col, col.description)
+            yield col.description.get_node(node_key)
+
+        if col.badges:
+            col_badge_metadata = BadgeMetadata(
+                start_label=ColumnMetadata.COLUMN_NODE_LABEL,
+                start_key=self._get_col_key(col),
+                badges=col.badges)
+            badge_nodes = col_badge_metadata.get_badge_nodes()
+            for node in badge_nodes:
+                yield node
+
+        type_metadata = col.get_type_metadata()
+        if type_metadata:
+            yield from type_metadata.create_node_iterator()
+
     def create_next_relation(self) -> Union[GraphRelationship, None]:
         try:
             return next(self._relation_iterator)
@@ -609,31 +502,7 @@ class TableMetadata(GraphSerializable, TableSerializable, AtlasSerializable):
                 yield tag_relationship
 
         for col in self.columns:
-            column_relationship = GraphRelationship(
-                start_label=TableMetadata.TABLE_NODE_LABEL,
-                start_key=self._get_table_key(),
-                end_label=ColumnMetadata.COLUMN_NODE_LABEL,
-                end_key=self._get_col_key(col),
-                type=TableMetadata.TABLE_COL_RELATION_TYPE,
-                reverse_type=TableMetadata.COL_TABLE_RELATION_TYPE,
-                attributes={}
-            )
-            yield column_relationship
-
-            if col.description:
-                yield col.description.get_relation(
-                    ColumnMetadata.COLUMN_NODE_LABEL,
-                    self._get_col_key(col),
-                    self._get_col_description_key(col, col.description)
-                )
-
-            if col.badges:
-                badge_metadata = BadgeMetadata(start_label=ColumnMetadata.COLUMN_NODE_LABEL,
-                                               start_key=self._get_col_key(col),
-                                               badges=col.badges)
-                badge_relations = badge_metadata.get_badge_relations()
-                for relation in badge_relations:
-                    yield relation
+            yield from self._create_column_relations(col)
 
         others = [
             GraphRelationship(
@@ -661,6 +530,37 @@ class TableMetadata(GraphSerializable, TableSerializable, AtlasSerializable):
                 TableMetadata.serialized_rels_keys.add((rel_tuple.start_key, rel_tuple.end_key, rel_tuple.type))
                 yield rel_tuple
 
+    def _create_column_relations(self, col: ColumnMetadata) -> Iterator[GraphRelationship]:
+        column_relationship = GraphRelationship(
+            start_label=TableMetadata.TABLE_NODE_LABEL,
+            start_key=self._get_table_key(),
+            end_label=ColumnMetadata.COLUMN_NODE_LABEL,
+            end_key=self._get_col_key(col),
+            type=TableMetadata.TABLE_COL_RELATION_TYPE,
+            reverse_type=TableMetadata.COL_TABLE_RELATION_TYPE,
+            attributes={}
+        )
+        yield column_relationship
+
+        if col.description:
+            yield col.description.get_relation(
+                ColumnMetadata.COLUMN_NODE_LABEL,
+                self._get_col_key(col),
+                self._get_col_description_key(col, col.description)
+            )
+
+        if col.badges:
+            badge_metadata = BadgeMetadata(start_label=ColumnMetadata.COLUMN_NODE_LABEL,
+                                           start_key=self._get_col_key(col),
+                                           badges=col.badges)
+            badge_relations = badge_metadata.get_badge_relations()
+            for relation in badge_relations:
+                yield relation
+
+        type_metadata = col.get_type_metadata()
+        if type_metadata:
+            yield from type_metadata.create_relation_iterator()
+
     def create_next_record(self) -> Union[RDSModel, None]:
         try:
             return next(self._record_iterator)
