@@ -1,26 +1,25 @@
 # Copyright Contributors to the Amundsen project.
 # SPDX-License-Identifier: Apache-2.0
 
-from pyhocon import ConfigTree  # noqa: F401
-from typing import Dict, Optional  # noqa: F401
+from typing import Optional, Tuple
+
+from pyhocon import ConfigTree
 
-from databuilder.transformer.base_transformer import Transformer
-from databuilder.models.table_column_usage import ColumnReader, TableColumnUsage
 from databuilder.extractor.bigquery_usage_extractor import TableColumnUsageTuple
+from databuilder.models.table_column_usage import ColumnReader, TableColumnUsage
+from databuilder.transformer.base_transformer import Transformer
 
 
 class BigqueryUsageTransformer(Transformer):
 
-    def init(self, conf):
-        # type: (ConfigTree) -> None
+    def init(self, conf: ConfigTree) -> None:
         """
         Transformer to convert TableColumnUsageTuple data to bigquery usage data
         which can be uploaded to Neo4j
         """
         self.conf = conf
 
-    def transform(self, record):
-        # type: (Dict) -> Optional[TableColumnUsage]
+    def transform(self, record: Tuple[TableColumnUsageTuple, int]) -> Optional[TableColumnUsage]:
         if not record:
             return None
 
@@ -40,6 +39,5 @@ class BigqueryUsageTransformer(Transformer):
 
         return TableColumnUsage(col_readers=col_readers)
 
-    def get_scope(self):
-        # type: () -> str
+    def get_scope(self) -> str:
         return 'transformer.bigquery_usage'
